;; Generated package description from evil-collection.el  -*- mode: lisp-data; no-byte-compile: t; lexical-binding:t -*-
(define-package "evil-collection" "3.0.1" "A set of keybindings for Evil mode" '((emacs "29.1") (evil "1.2.13")) :commit "fa8da0ebba4bbf2a84a78183420d8303179ef427" :authors '(("James Nguyen" . "james@jojojames.com")) :maintainer '("James Nguyen" . "james@jojojames.com") :keywords '("evil" "tools") :url "https://github.com/emacs-evil/evil-collection")
